/**
  *  \file client/chart/drawmarkermode.cc
  *  \brief Starchart Mode: WDrawMarkerChartMode
  *
  *  (c) 2001-2013 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/chart/drawmarkermode.h"
#include "client/chartloc.h"
#include "client/dialogs/helpdlg.h"
#include "game/global.h"
#include "gfx/gfxcontext.h"
#include "gfx/palette.h"
#include "ui/event.h"
#include "ui/uiglobal.h"
#include "util/nls.h"

WDrawMarkerChartMode::WDrawMarkerChartMode(GDrawingContainer::iterator p)
    : WMarkerChartMode(p)
{ }

WDrawMarkerChartMode::~WDrawMarkerChartMode()
{
    finishDrawing();
}

void
WDrawMarkerChartMode::finishDrawing()
{
    GDrawingContainer& c = getDisplayedTurn().getCurrentUniverse().getDrawings();
    if (*drawing) {
        if ((*drawing)->getPos() == (*drawing)->getPos2()) {
            c.erase(drawing);
        }
    }
}

void
WDrawMarkerChartMode::onMove(GChartLocation& loc)
{
    if (*drawing) {
        GPoint adj = loc.getWrapAdjust();
        if (adj.x || adj.y) {
            (*drawing)->setPos((*drawing)->getPos() + adj);
            loc.markChanged();
        }
        if ((*drawing)->getPos2() != loc.getLocation()) {
            (*drawing)->setPos2(loc.getLocation());
            loc.markChanged();
        }
    }
    WMarkerChartMode::onMove(loc);
}

void
WDrawMarkerChartMode::drawOverlays(GfxCanvas& can, GChartLocation& loc)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(GfxStandardColorScheme::instance);
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setColor(COLOR_WHITE);

    GfxPoint sc = loc.getViewport().view.getTopLeft();
    outText(ctx, sc.x, sc.y, _("Mark 2nd point, end with [ESC] or [+]."));

    WMarkerChartMode::drawOverlays(can, loc);
}

bool
WDrawMarkerChartMode::handleEvent(const UIEvent& event, GChartLocation& loc)
{
    if (event.type == ev_Key) {
        switch (event.key.code) {
         case '+':
            if (*drawing && (*drawing)->getType() == LineDrawing) {
                // Line: start new line
                if ((*drawing)->getPos() != (*drawing)->getPos2()) {
                    GDrawing nd(**drawing);
                    nd.setPos((*drawing)->getPos2());
                    nd.setPos2((*drawing)->getPos2());
                    drawing = getDisplayedTurn().getCurrentUniverse().getDrawings().add(nd);
                    loc.markChanged();
                }
            } else {
                // Rectangle: exit move mode (or marker deleted)
                delete this;
            }
            return true;
         case '-':
            // No effect, because we handle '+' specially. Avoids that users
            // zoom out and cannot zoom in.
            return true;
         case SDLK_BACKSPACE:
            if (*drawing && (*drawing)->getPos() != (*drawing)->getPos2()) {
                // Go to start point
                loc.setLocation((*drawing)->getPos());
            } else {
                // Trying to backspace from zero-size drawing, so exit mode
                delete this;
            }
            return true;
         case 'x':
            // Go to start point, swapping beginning and end of drawing
            if (*drawing) {
                GPoint a = (*drawing)->getPos(), b = (*drawing)->getPos2();
                (*drawing)->setPos(b);
                (*drawing)->setPos2(a);
                loc.setLocation(a);
            }
            return true;
         case 'p':
            delete this;
            return true;
         case 'h':
         case ss_Alt + 'h':
            doHelp("pcc2:draw");
            return true;
        }
    }
    return WMarkerChartMode::handleEvent(event, loc);
}
