/**
  *  \file game/formula.cc
  *  \brief Math support
  *
  *  Note that this module should be strictly independant and have
  *  no dependencies on the rest of PCC2, because it is used in FLAK.
  *
  *  (c) 2001-2013 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <cmath>
#include "game/formula.h"

/** Pi. This is what "echo 'a(1)*4' | bc -l" says. */
const double PI = 3.14159265358979323844;

/** Arithmetic rounding. Rounds towards nearest integer, except when
    number ends in .5 exactly where it rounds to next larger integer
    (where -10 is larger than -9).

    This function used to be called /round/ but it seems SUSv3 defines
    such a function, too. */
int32
roundToInt(double d)
{
    if (d < 0)
        return -int(-d + 0.5);
    else
        return int(d + 0.5);
}

/** Given coordinate offsets, compute heading.
    \pre dx!=0 || dy!=0
    \returns heading in radians, [0, 2*PI] */
double
getHeadingRad(double dx, double dy)
{
    // atan2 returns [-PI, PI], says C99. We want it to return [0, 2*PI].
    // Yes, this ought to be dx,dy, to get the angles in the way VGAP uses them.
    double value = std::atan2(dx, dy);
    if (value < 0)
        value += 2*PI;
    return value;
}
