/**
  *  \file int/if/objif.h
  *  \brief Interpreter Game Interface: Common Object Methods
  *
  *  (c) 2001-2013 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_OBJIF_H
#define PCC_V2_INT_IF_OBJIF_H

#include "int/compound.h"
#include "int/value.h"

class GObject;
class GUniverse;

/** Object method value. Similar to IntSimpleProcedureValue, but also
    keeps a GObject reference which it hands to the procedure when
    called. */
class IntObjectProcedureValue : public IntCallableValue {
 public:
    typedef void (*call_t)(GObject& obj, GUniverse& univ, IntExecutionContext& exc, IntArgBlock& args);

    IntObjectProcedureValue(GObject& obj, GUniverse& univ, call_t call);
    ~IntObjectProcedureValue();

    // IntCallableValue:
    void      call(IntExecutionContext& exc, IntDataSegment& args, bool want_result);
    bool      isProcedureCall();
    int32_t   getDimension(int32_t);
    IntContext* makeFirstContext();

    // IntValue:
    string_t    toString(bool readable);
    void        store(IntTagNode&, Stream&);
    IntValue*   clone();

 private:
    GObject& obj;
    GUniverse& univ;
    call_t call_function;
};

void IFObjMark(GObject& obj, GUniverse& univ, IntExecutionContext& exc, IntArgBlock& args);
void IFObjUnmark(GObject& obj, GUniverse& univ, IntExecutionContext& exc, IntArgBlock& args);

#endif
